#synthDrivers/dtusbd.py
#A part of NonVisual Desktop Access (NVDA)
#copyright 2011 by NVDA contributers etc
#www.nvda-project.org/
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.
#this thing is responsive. Watch out!

from cStringIO import StringIO
import synthDriverHandler
from synthDriverHandler import SynthDriver,VoiceInfo
from collections import OrderedDict
from logHandler import log
from ctypes import *
from ctypes.wintypes import *
import config
import speech
from winUser import WNDCLASSEXW, WNDPROC
#import tones
import logging
import re

usbdt = cdll.dtusbd
WM_INDEX = 0x401
lastIndex = None
chars = []
index_re = re.compile(r'\[:index (\d+)\]')
@WNDPROC
def nvdaDtUSBWndProc(hwnd, msg, wParam, lParam):
	"""Indexes are sent, one byte per message, as [:index %d]."""
	global lastIndex, chars
	if msg == WM_INDEX:
		chars.append(chr(wParam))
		if chars[-1] == ']':
			s = "".join(chars)
			m = index_re.search(s)
			chars = []
			if m:
				lastIndex = int(m.group(1))
		return 0
	else:
		return windll.user32.DefWindowProcW(hwnd,msg,wParam,lParam)

appInstance = windll.kernel32.GetModuleHandleW(None)
nvdaDtUSBWndCls = WNDCLASSEXW()
nvdaDtUSBWndCls.cbSize = sizeof(nvdaDtUSBWndCls)
nvdaDtUSBWndCls.lpfnWndProc = nvdaDtUSBWndProc
nvdaDtUSBWndCls.hInstance = appInstance
nvdaDtUSBWndCls.lpszClassName = u"nvdaDtUSBWndCls"

class SynthDriver(synthDriverHandler.SynthDriver):
	name="dtusb"
	description=_("DECtalk USB")
	supportedSettings=(SynthDriver.VoiceSetting(),SynthDriver.RateSetting(),SynthDriver.PitchSetting(),SynthDriver.InflectionSetting())
	minRate = 75
	maxRate = 650
	minPitch=50
	maxPitch=350
	minInflection=0
	maxInflection=250

#Runs via the USB port; a DECtalk Express driver may happen but for now I'm just doing USB.
	@classmethod
	def check(cls):
		usbdt.USBDT_CloseSynth()
		usbdt.USBDT_OpenSynth()
		return usbdt.USBDT_ReadyCheck()

		

	def __init__(self):
		super(synthDriverHandler.SynthDriver,self).__init__()
		self.dt_rate = 300
		self.dt_pitch=100
		self.dt_inflection=100
		self.voice='Paul'
		self._messageWindowClassAtom = windll.user32.RegisterClassExW(byref(nvdaDtUSBWndCls))
		self._messageWindow = windll.user32.CreateWindowExW(0, self._messageWindowClassAtom, u"nvdaDtUsbWndCls window", 0, 0, 0, 0, 0, None, None, appInstance, None)
		usbdt.USBDT_RegisterWindowIndexes(self._messageWindow, WM_INDEX)
		usbdt.USBDT_ResumeIndexes(1)

	def speak(self,speechSequence):
		text_list = []
		for item in speechSequence:
			if isinstance(item, basestring):
				text_list.append(item)
			elif isinstance(item, speech.IndexCommand):
				text_list.append(u"[:i r %d]" % item.index)


		text = " ".join(text_list).encode('iso8859-1', 'ignore')+'\13'
		#prevent control strings from going into our text from input
#		text = text.replace('[:', ' ') #keep this commented out as it messes with indexes for some reason...
		if text:
			text = "[:dv gf 65 ri 90 ap %d] %s" % (self.dt_pitch, text)
			text = "[:dv pr %d] %s" % (self.dt_inflection, text)
			voice = self._voice[0].lower()
			text = "[:Name %s] %s" % (self.voice, text)
			text = "[:ra %d] %s" % (self.dt_rate, text)
			usbdt.USBDT_WriteString(text, len(text))

	def cancel(self):
		usbdt.USBDT_ShutUp()
		usbdt.USBDT_ResumeSpeech() #make sure speech is always unpaused after shutting up.

	def _set_rate(self, rate):
		val = self._percentToParam(rate, self.minRate, self.maxRate)
		self.dt_rate = val

	def _get_rate(self):
		return self._paramToPercent(self.dt_rate, self.minRate, self.maxRate)

	def _set_pitch(self, pitch):
		val = self._percentToParam(pitch, self.minPitch, self.maxPitch)
		self.dt_pitch = val

	def _get_pitch(self):
		return self._paramToPercent(self.dt_pitch, self.minPitch, self.maxPitch)

	def _set_inflection(self, inflection):
		val = self._percentToParam(inflection, self.minInflection, self.maxInflection)
		self.dt_inflection = val

	def _get_inflection(self):
		return self._paramToPercent(self.dt_inflection, self.minInflection, self.maxInflection)

	def _get_lastIndex(self):
		return lastIndex

	def _get_availableVoices(self):
		voices = OrderedDict()
		for i in 'Paul Betty Harry Frank Dennis Kit Ursula Rita Wendy'.split(' '):
			voices[i] = VoiceInfo(i, i)
		return voices

	def _get_voice(self):
		return self._voice

	def _set_voice(self, val):
		self._voice = str(val)

	def terminate(self):
		usbdt.USBDT_RegisterWindowIndexes(0, WM_INDEX)
		usbdt.USBDT_CloseSynth()
		windll.user32.DestroyWindow(self._messageWindow)
		windll.user32.UnregisterClassW(self._messageWindowClassAtom,appInstance)

	def pause(self,switch):
		pausething="[:pp -20000][:cp -20000]"
		if switch:
			usbdt.USBDT_PauseSpeech()
		else:
			usbdt.USBDT_ResumeSpeech()
			usbdt.USBDT_WriteString(pausething,len(pausething))
